//+------------------------------------------------------------------+
//|                                           AbsoluteStrengthEA.mq5 |
//|                                   Hakan SARI sistemliborsaci.com |
//|                                  https://www.sistemliborsaci.com |
//+------------------------------------------------------------------+
#property copyright "Hakan SARI sistemliborsaci.com"
#property link      "https://www.sistemliborsaci.com"
#property version   "3.1"


#include <trade/trade.mqh>

input int trend = 0;
int ea_magic = 0;
input double ea_lot = 0.01;
input int ea_sl = 0;
input int ea_tp = 0;

input bool ea_ters_sinyal_kapat = true;

enum ENUM_AS_MODE
{
   RSI_method,        // Calculate absolute strength using RSI method
   Stochastic_method, // Calculate absolute strength using Stochastic method
   ADX_method         // Calculate absolute strength using ADX method
};
enum enAverages
{
   avgSma,    // Simple moving average
   avgEma,    // Exponential moving average
   avgDsema,  // Double smoothed EMA
   avgDema,   // Double EMA
   avgTema,   // Tripple EMA
   avgSmma,   // Smoothed MA
   avgLwma,   // Linear weighted MA
   avgPwma,   // Parabolic weighted MA
   avgAlex,   // Alexander MA
   avgVwma,   // Volume weighted MA
   avgHull,   // Hull MA
   avgTma,    // Triangular MA
   avgSine,   // Sine weighted MA
   avgLsma,   // Linear regression value
   avgIe2,    // IE/2
   avgNlma,   // Non lag MA
   avgZlma,   // Zeo lag EMA
   avgLead    // Leader EMA
};
enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average     // Average (high+low+oprn+close)/4
};


input ENUM_AS_MODE Mode       = RSI_method;    // Calculation method
input int          Length     = 2;            // Calculation length
input int          Smooth     = 3;             // Smoothing length
input int          Signal     = 1;             // Signal smoothing length
input enPrices     Price      = pr_close;      // Price
input enAverages   AvgMethod  = avgAlex;       // Averaging method
input bool   arrowsVisible    = true;         // Arrows visible?
input string arrowsIdentifier = "absArrows1";  // Unique ID for arrows
input double arrowsUpperGap   = 0.5;           // Upper arrow gap
input double arrowsLowerGap   = 0.1;           // Lower arrow gap
input color  arrowsUpColor    = clrLimeGreen;  // Up arrow color
input color  arrowsDnColor    = clrOrange;     // Down arrow color
input int    arrowsUpCode     = 225;           // Up arrow code
input int    arrowsDnCode     = 226;           // Down arrow code
input int    arrowsSize       = 1;             // Arrows size

input int gecikme = 0;

input bool cci_aktif = true;

input ENUM_TIMEFRAMES cci_timeframe = 0;
input int cci_period = 14;
input ENUM_APPLIED_PRICE cci_applied_price = 5;
input int cci_gecikme = 1;

input bool macd_aktif = false;
input ENUM_TIMEFRAMES macd_timeframe = 0;
input int macd_fast = 4;
input int macd_slow = 6;
input int macd_signal = 4;
input ENUM_APPLIED_PRICE macd_price = PRICE_CLOSE;
input int macd_gecikme = 1;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+

int asi;
int icci;

int imacdi;

int OnInit()
  {
//---

   asi = iCustom(_Symbol,_Period,"AbsoluteStrength",Mode,Length,Smooth,Signal,Price,AvgMethod,arrowsVisible,
   arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize);
   if(cci_aktif) icci = iCCI(_Symbol,cci_timeframe,cci_period,cci_applied_price);
   if(macd_aktif) imacdi = iMACD(_Symbol,macd_timeframe,macd_fast,macd_slow,macd_signal,macd_price);
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+

datetime last_signal;
datetime last_close;

int last_trend = 0;

void OnTick()
  {
//---
   
   double smoothbull[],smoothbear[],signalbull[],signalbear[];
   

   ArraySetAsSeries(smoothbull, true);
   CopyBuffer(asi,0,0,20,smoothbull);
   
   ArraySetAsSeries(smoothbear, true);
   CopyBuffer(asi,1,0,20,smoothbear);
   
   ArraySetAsSeries(signalbull, true);
   CopyBuffer(asi,2,0,20,signalbull);
   
   ArraySetAsSeries(signalbear, true);
   CopyBuffer(asi,3,0,20,signalbear);
   
   double son_cci[];
   double son_macd[],son_macd_signal[];
   
   if(cci_aktif) {   
   
   ArraySetAsSeries(son_cci, true);
   CopyBuffer(icci,0,0,20,son_cci);
   }
   
   if(macd_aktif) {
   
   ArraySetAsSeries(son_macd, true);
   CopyBuffer(imacdi,0,0,20,son_macd);
   
   ArraySetAsSeries(son_macd_signal, true);
   CopyBuffer(imacdi,1,0,20,son_macd_signal);
   }
   
   double ask=SymbolInfoDouble(Symbol(),SYMBOL_ASK); 
   double bid=SymbolInfoDouble(Symbol(),SYMBOL_BID);   
   
   datetime jtime = iTime(_Symbol,_Period,0);
     
   bool smt_buy = (gecikme>0 && smoothbull[gecikme]>smoothbear[gecikme] && smoothbull[gecikme+1]<=smoothbear[gecikme+1]) || (gecikme==0 && smoothbull[gecikme]>smoothbear[gecikme]);
   bool smt_sell = (gecikme>0 && smoothbull[gecikme]<smoothbear[gecikme] && smoothbull[gecikme+1]>=smoothbear[gecikme+1]) || (gecikme==0 && smoothbull[gecikme]<smoothbear[gecikme]);
     
   bool buy_signal = smt_buy && last_signal<jtime && (trend == 0 || trend == 1) && ( cci_aktif == false || (cci_aktif == true && son_cci[cci_gecikme]>0) )    && ( macd_aktif == false || (macd_aktif == true && son_macd[macd_gecikme]>son_macd_signal[macd_gecikme] && son_macd[macd_gecikme+1]<son_macd_signal[macd_gecikme+1]  ) ) ;  
   bool sell_signal = smt_sell && last_signal<jtime && (trend == 0 || trend == -1) && ( cci_aktif == false || (cci_aktif == true && son_cci[cci_gecikme]<0) )   && ( macd_aktif == false || (macd_aktif == true && son_macd[macd_gecikme]<son_macd_signal[macd_gecikme] && son_macd[macd_gecikme+1]>son_macd_signal[macd_gecikme+1]  ) ) ;  
     
   if(buy_signal ) {
      if(ea_ters_sinyal_kapat) hepsinikapat(1);
      last_close = jtime;
   } else if(sell_signal ) {
      if(ea_ters_sinyal_kapat) hepsinikapat(0);
      last_close = jtime;
   }
   
   if(buy_signal && ( last_trend == 0 || last_trend == -1 ) && last_signal< jtime) {
      open_order(ORDER_TYPE_BUY, ea_lot, ask, (ea_sl>0 ? ask-ea_sl*_Point : 0),  (ea_tp>0 ? ask+ea_tp*_Point : 0) );
      last_signal = jtime;
      last_trend = 1;
   } else if(sell_signal && ( last_trend == 0 || last_trend == 1 ) && last_signal< jtime ) {
      open_order(ORDER_TYPE_SELL, ea_lot, bid, (ea_sl>0 ? bid+ea_sl*_Point : 0),  (ea_tp>0 ? bid-ea_tp*_Point : 0) );
      last_signal = jtime;
      last_trend = -1;
   }
   
   
   string cmd = "";
   StringConcatenate(cmd,cmd,"\n","version = 1.3");
   StringConcatenate(cmd,cmd,"\n","trend = ",trend);
   StringConcatenate(cmd,cmd,"\n","ea_lot = ",ea_lot);
   StringConcatenate(cmd,cmd,"\n","ea_sl = ",ea_sl);
   StringConcatenate(cmd,cmd,"\n","ea_tp = ",ea_tp);
   StringConcatenate(cmd,cmd,"\n","ea_ters_sinyal_kapat = ",ea_ters_sinyal_kapat);
   StringConcatenate(cmd,cmd,"\n","");
   StringConcatenate(cmd,cmd,"\n","Absolute Strength ( Length:",Length," , Smooth:",Smooth," , Signal:",Signal,")");
   StringConcatenate(cmd,cmd,"\n","smoothbull[",gecikme,"] = ",smoothbull[gecikme]);
   StringConcatenate(cmd,cmd,"\n","smoothbear[",gecikme,"] = ",smoothbear[gecikme]);
   StringConcatenate(cmd,cmd,"\n","signalbull[",gecikme+1,"] = ",signalbull[gecikme+1]);
   StringConcatenate(cmd,cmd,"\n","signalbear[",gecikme+1,"] = ",signalbear[gecikme+1]);
   if(cci_aktif) StringConcatenate(cmd,cmd,"\n","son_cci[",gecikme,"] = ",son_cci[gecikme]);
   if(macd_aktif) StringConcatenate(cmd,cmd,"\n","son_macd[",gecikme,"] = ",son_macd[gecikme]);
   if(macd_aktif) StringConcatenate(cmd,cmd,"\n","son_macd_signal[",gecikme,"] = ",son_macd_signal[gecikme]);
   StringConcatenate(cmd,cmd,"\n","jtime = ",jtime);
   StringConcatenate(cmd,cmd,"\n","last_signal = ",last_signal);


   Comment(cmd);   
   
   
   

   
  }



void hepsinikapat(int ktype)

{

   string      type;

   CTrade trade;
   for (int i=PositionsTotal()-1;i>=0; i--) 
   { 
   
   

      
      
      if(PositionGetSymbol(i) == _Symbol){       
      
      type = EnumToString( (ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE));
   
         if ( (ktype == 0 && type == "POSITION_TYPE_BUY") || (ktype == 1 && type == "POSITION_TYPE_SELL") ) {
                
            if(!trade.PositionClose(PositionGetSymbol(i))) 
            {
               Print(PositionGetSymbol(i), " PositionClose() method failed. Return code=",trade.ResultRetcode(),
               ". Code description: ",trade.ResultRetcodeDescription());
            } else {
               Print(PositionGetSymbol(i), " PositionClose() method executed successfully. Return code=",trade.ResultRetcode(),
               " (",trade.ResultRetcodeDescription(),")");
            }
         
         }
         
      }

   }

     

} 






void hepsinikapat2() {
   
   CTrade trade;
      
   for (int i = PositionsTotal()-1;i>=0;i--) {
   
      
   
      if( PositionGetSymbol(i) != _Symbol ) continue;
      
      
      
      if (trade.PositionClose(PositionGetSymbol(i))) { }
   }
   
   DeleteAllOrders();   
   
   
}




void DeleteAllOrders() 
  { 
   ulong order_ticket; 
//--- go through all pending orders 
   for(int i=OrdersTotal()-1;i>=0;i--)  {
      if((order_ticket=OrderGetTicket(i))>0)  {
         //--- order with appropriate ORDER_MAGIC 

      if( OrderGetString(ORDER_SYMBOL) != _Symbol ) continue;

      MqlTradeResult result={0}; 
      MqlTradeRequest request={0}; 
      request.order=order_ticket; 
      request.action=TRADE_ACTION_REMOVE; 
      int bg = OrderSend(request,result); 
      //--- write the server reply to log 
      Print(__FUNCTION__,": ",result.comment," reply code ",result.retcode); 
      
      
      } }
//--- 
  }



ulong onceki_ticket = 0;
ulong last_ticket = 0;
  
input int slippage = 10;  
  
void open_order(ENUM_ORDER_TYPE otype, double lot, double fiyati, double sl, double tp ) {
      
      CTrade MyTrade;

      // MyTrade.SetExpertMagicNumber(magic);
      MyTrade.SetDeviationInPoints(slippage);
      bool ret=MyTrade.PositionOpen(_Symbol,otype,lot,fiyati,sl,tp,"");


      /*
      MqlTradeRequest request={0}; 
      request.action=TRADE_ACTION_DEAL;
      request.magic=ea_magic;
      request.symbol=_Symbol;
      request.volume=lot; 
      request.sl = sl;
      request.tp = tp;    

      request.type= otype;
   
      request.price= fiyati;
  
      MqlTradeResult result={0}; 
      int gbd = OrderSend(request,result);  

      // 

      */
      
      onceki_ticket = last_ticket;
      last_ticket = MyTrade.ResultOrder();      
      Print("last_ticket : ",last_ticket," onceki_ticket = ",onceki_ticket);

}
  
  
datetime Time[];
  
datetime iTime(string symbol,ENUM_TIMEFRAMES timeframe,int index)
  {
 datetime etime=0;
 ArraySetAsSeries(Time,true);
 int copied=CopyTime(symbol,timeframe,0,2000,Time);
 if(copied>0 && index<copied) etime=Time[index];
 return(etime);
  }
  